; Target: PIC12F629
;
    list	P = PIC12F629
#include "p12f629.inc"
;
    errorlevel	-302
;
; __config 0x31B4
 __CONFIG _FOSC_INTRCIO & _WDTE_OFF & _PWRTE_OFF & _MCLRE_ON & _BOREN_OFF & _CP_OFF & _CPD_OFF
;
; Internal oscillator is 4mhz. Fosc/4 => instruction cycle ~1us (1mhz)
; leave the defaults for clock speed
;
;	       -------
;	vdd ==|1     8|== GND
;	GP5 <=|2     7|== GP0 / ICSP DAT
;       GP4 =>|3     6|<= GP1 / ICSP CLK
;  GP3/MCLR ==|4     5|=> GP2
;	       -------
;
; GP0 (7) : ICSP DAT
; GP1 (6) : ICSP CLK / Zero Crossing sense (input)
; GP2 (5) : Triac trigger (output)
; GP3 (4) : ICSP MCLR
; GP4 (3) : Power push button switch (input)
; GP5 (2) : Power push button LED (output)
;
; (2) inputs:
; GP1 (6) : Zero Cross Sense
; GP4 (3) : Push button switch
;
; (2) outputs:
; GP2 (5) : Triac Trigger
; GP5 (2) : Button LED
;
; ICSP reserved pins: (used only for ICSP)
; GP0/DAT  (7)
; GP3/MCLR (4)
;
; Shared pins with ICSP:
; GP1/CLK  (6)
;
; LED on GP5 is pulled up via res to Vdd => GP5 @ 0 to light LED up
;
; data ram (64bytes) @ h20-h5F accessed from both banks (hA0-hDF accesses h20-h5F)
;
    cblock	0x20
;
Semaphores
Kount	    ; soft start sequence half wave per stage counter
Kount250us  ; delay routine counter
Kount10ms
Kount1s
Kountvar    ; variable delay routine counter for phase delay
KountSTG    ; soft start sequence stage counter
KountTrain  ; triac pulse train counter
Sav_Kount   ; used to save the current pulse train count during loop
Sav_W	    ; ISR W reg save
Sav_STAT    ; ISR STATUS reg save
;
    endc
;
; GPIO in bank 0
;
#define	    Z_CROSS	GPIO,1	    ; zero cross sense on GP1 (input)
#define	    Triac	GPIO,2	    ; Triac trigger is on GP2 (output)
#define	    PWR_BUTT	GPIO,4	    ; Power push button switch on GP4 (input)
#define	    PWR_LED	GPIO,5	    ; Power button LED is on GP5 (output)
;
#define	    ZCD	    Semaphores,0    ; zero-cross sensed flag

;
NUMHW	    EQU	    d'16'   ; number of half waves to count per soft start stage
NUMSTG	    EQU	    d'18'   ; total number of stages in startup sequence
			    ; 500us increments x 18 => 9ms
;
;*******************************************************************************
;
    org	    0	    ; start program @ reset
    goto    init
;
;*******************************************************************************
;
    org	    4	    ; interrupt handling ISR
;
; first, save environment context
;
    movwf   Sav_W	; save current W
    swapf   STATUS,w	; move status register into W (use swap to not alter afterwards)
    movwf   Sav_STAT	; save STATUS (nibbles are swapped)
;
; we can only come here when a zero-crossing event occured
;
    bsf	    Z_CROSS	; clear the condition first (by making a write to it)
    bcf	    INTCON,GPIF	; then clear the IOC flag
    bsf	    ZCD		; signal that zero-crossing happened
;
; restore environment context before leaving
;
    swapf   Sav_STAT,w	; retrieve STATUS (nibbles were swapped when saved)
    movwf   STATUS	; restore pre-isr STATUS register
    swapf   Sav_W,f	; use swaps to not alter the Z status flag (nibbles swapped)
    swapf   Sav_W,w	; restore pre-isr W register (swap back the nibbles)
    retfie		; return from interrupt
;
;*******************************************************************************
;
; initializations
;
init
;
    bsf	    STATUS,RP0	; bank 1
    clrf    PIE1	; no interrupts (don't start sensing for zero-cross yet)
    clrf    INTCON	; (reset clears this)
    bcf	    OPTION_REG,NOT_GPPU	; Activate weak pull-ups
;
    bcf	    STATUS,RP0	; bank 0
    clrf    GPIO
    clrf    Semaphores
;
; comparator off => digital I/Os
;
    movlw   b'00000111'	; comp off >> dig I/Os
    movwf   CMCON
    bsf	    STATUS,RP0	; bank 1
    movlw   b'00010010'	; GP1 & GP4 as inputs
    movwf   TRISIO	; all others as outputs
    movlw   b'00010010'	; turn on weak pull ups for GP1 & GP4
    movwf   WPU
    movlw   b'00000010'
    movwf   IOC		; and on-change interrupts are for GP1 only
;
    call    Delay1s	; give a bit of settling time
    call    LEDWASON	; turn off the power button LED
;
;*******************************************************************************
;
; At initial state with mains powered, unit is in standby
; just waiting for power push button to be pressed
; also returns to this status when unit was on but was turned off
;
;*******************************************************************************
;
StandByLoop
;
    bcf	    STATUS,RP0	; bank 0
    btfsc   PWR_BUTT	; check for pressed power push button
			; button pressed gives a 0 (weak pull-up enabled on GP4)
    goto    $-1
    call    Delay10ms	; do some debouncing
    bcf	    STATUS,RP0	; bank 0
    btfsc   PWR_BUTT
    goto    $-5
    call    Delay10ms	; some more debouncing
    bcf	    STATUS,RP0	; bank 0
    btfsc   PWR_BUTT
    goto    $-9
;
; here, we are certain the power push button was pressed, so we go on to the startup sequence
;
    bsf	    INTCON,GIE	; turn on interrupts
    bsf	    INTCON,GPIE	; and turn on IOC for GP1
    bcf	    INTCON,GPIF	; for good measure, start from a cleared int flag
    call    Delay1s	; give a bit of wait time
    call    LEDWASOFF	; turn on power push button LED
    movlw   NUMSTG	; initialize startup sequence stage total number (18)
    movwf   KountSTG	; value used for delay/phase angle and triac pulse train
    movlw   NUMHW	; and the number of half waves count per stage (16)
    movwf   Kount	; used for iteration through set number of half waves
    clrf    KountTrain	; prepare to set up the pulse train counter ...
    incf    KountTrain,1    ; with 1 initial pulse in train
    bcf	    ZCD		; clear any previous zero-cross event (to prevent short unwanted pulse)
;
;*******************************************************************************
;
; This is the startup sequence loop
; while this loop runs, we don't poll the power button switch
; we blink/toggle the power LED until all done, then turn it on fully
; then move on to the main loop and start polling the power button
;
StartUpLoop
;
    btfss   ZCD		; zero-crossing occurred? (was set by ISR)
    goto    $-1
;
; here, we had zero-cross sensed
;
    bcf	    ZCD		; clear the zero-cross event
;
; here, we go through the ramp up sequence for each half wave
;
    movf    KountSTG,w	; take current stage count as multiplier for
    movwf   Kountvar	; (phase) delay (initially 9ms, and decreasing)
    call    DelayVar	; then call up for delay
    call    PulseTRIAC	; and pulse (train) the Triac (250us @ 50% duty)
    decfsz  Kount	; count down the half waves
    goto    StartUpLoop	; if we're not done with this current stage, just loop
;
; here, we're done with a half wave full count in sequence
;
    movlw   NUMHW	; when down to 0, re-initialize it for next stage (if any)
    movwf   Kount	; (number of half wave in sequence stage)
    call    TOGGLE_LED	; blink the power LED
    incf    KountTrain	; one more pulse in next train
    decfsz  KountSTG	; count down sequence
    goto    StartUpLoop	; if not over yet, continue looping
;
; here, we're done with the sequence countdown
;
    bcf	    STATUS,RP0	; bank 0
    bcf	    INTCON,GIE	; turn off interrupts
    bsf	    Triac	; permanently turn on the Triac
    call    Delay1s	; give it a little time
    call    LEDWASOFF	; turn on power LED before moving on to the main loop
;
;*******************************************************************************
;
; This loop is for fully powered operation, just waiting for power push button
; to be pressed to power down, w
; (Triacs were fully turned on before coming here, and power button LED)
;
MainLoop
;
    bcf	    STATUS,RP0	; bank 0
    btfsc   PWR_BUTT	; check for pressed power push button
    goto    $-1
    call    Delay10ms	; do some debouncing
    bcf	    STATUS,RP0	; bank 0
    btfsc   PWR_BUTT
    goto    $-5
    call    Delay10ms	; some more debouncing
    bcf	    STATUS,RP0	; bank 0
    btfsc   PWR_BUTT	; are we sure?
    goto    $-9
;
; here, we have a power button pushed, to power down
;
    call    LEDWASON	; turn off power button LED
    clrf    Semaphores	; clear status semaphores,
    bcf	    STATUS,RP0	; bank 0
    bcf	    Triac	; turn off Triac
    call    Delay1s	; add a little delay
;
; we must do debouncing and wait for power push button to no longer be pressed
;
    bcf	    STATUS,RP0	; bank 0
    btfss   PWR_BUTT	; check for pressed power push button
    goto    $-1		; loop until button released
    call    Delay10ms	; a little debouncing
    bcf	    STATUS,RP0	; bank 0
    btfss   PWR_BUTT	; and look again
    goto    $-5		; if not really released, look again
    call    Delay10ms	; and more deboucing, we need to be sure
    bcf	    STATUS,RP0	; bank 0
    btfss   PWR_BUTT	; are we sure?
    goto    $-9
    call    Delay1s	; and when finally released, we wait
    call    Delay1s	; a while longer
    goto    StandByLoop	; and go back to standby status
;
;*******************************************************************************
;
; Service Subroutines
;
TOGGLE_LED
;
; LED is on with IO @ 0
;
    bcf	    STATUS,RP0	    ; bank 0
    btfss   PWR_LED	    ; was the led turned off?
    goto    LEDWASON
;
LEDWASOFF
;
    bcf	    STATUS,RP0	    ; bank 0
    bcf	    PWR_LED	    ; if it was off, turn it on
    return
;
LEDWASON
;
    bcf	    STATUS,RP0	    ; bank 0
    bsf	    PWR_LED	    ; if it was on, turn it off
    return
;
; Triac trigger pulse train
; KountSTG is current sequence stage counter
; NUMSTG is total sequence stage number
; KountTrain is number of pulses in train
; pulses 250us/250us (on/off) @ 50% duty cycle
;
PulseTRIAC
;
; PhotoTriac fires with IO @ 1
; Triac trigger GP2 on GPIO,2 in bank 0
;
    bcf	    STATUS,RP0	    ; bank 0
    movf    KountTrain,w    ; current count of pulses in train
    movwf   Sav_Kount	    ; is saved, so it can be restored once the count down is done
    bsf	    Triac	    ; turn on
    call    Delay250us	    ; wait to make pulse
    bcf	    STATUS,RP0
    bcf	    Triac	    ; and turn off
    call    Delay250us	    ; untriggered portion => 50% duty cycle
    decfsz  KountTrain,1    ; count down the number of pulses in train
    goto    $-6		    ; if not done yet, loop back to turn triac on
;
    movf    Sav_Kount,w	    ; take the saved current count of pulses in train
    movwf   KountTrain	    ; and restore it, since it was destroyed by count down
    return
;
;==========================================================
;
; Delay Subroutines
;
; One instruction cycle consists of four oscillator periods
; oscillator frequency of 4 MHz default
; this gives a nominal instruction execution time of ~1us
; all routines give only approximate delay
;
; 250us delay needs 250 cycles
; the call adds 2cycles to the total count
; count loop starts at d81
;
Delay250us
;
    bcf		STATUS,RP0	; bank 0 (1 cyc)
;    movlw	d'81'		; (1 cyc)
    movlw	d'92'		; (tweaked by experiment!!! ?!?!?!?)
    movwf	Kount250us	; (1)
    decfsz	Kount250us,1	; 1 x 80 (don't count the 0)
    goto	$-1		; 2 x 80 (last iteration will just do a nop instead of goto)
    return			; (2) + 1 (nop when done count loop)
;
; ~500us delay, call delay 250us twice
;
; used for phase angle delay, times the 18 stages, not too critical, but must not add up too
; much beyond the 9ms initial phase angle delay, plus the 250us triac trigger pulse, to leave
; at least 200us of time after pulse before the next zero crossing event
;
Delay500us
;
    call    Delay250us
    call    Delay250us
    return
;
; ~10ms delay, call 40 times 250us delay
; timing a bit more than 10ms, but good enough. used for debouncing
;
Delay10ms 
;
    movlw	d'40'	    ; 40 x 250us
    movwf	Kount10ms
    call	Delay250us
    decfsz	Kount10ms,1
    goto	$-2	    ; loop back to the call line
    return
;
; ~1 sec delay, call 100 times 10ms delay
; a bit more than 1sec, but good enough for functional delays
;
Delay1s
;
    movlw	d'100'	    ; 100 x 10ms
    movwf	Kount1s
    call	Delay10ms
    decfsz	Kount1s,1
    goto	$-2	    ; loop back to the call line
    return
;
; variable delay, call X times 500us delay
; set count in Kountvar register before calling this.
; the bank was already selected when initializing it
;
DelayVar
;
    call    Delay500us
    decfsz  Kountvar,1
    goto    $-2		; loop back to the call line
    return
;
;======================================================
;
    END